
require 'net/sshr'
require 'term/ansicolor'
include Net::SSHR
include Term::ANSIColor

# = ABSTRACT
#
# Triage is a ruby module for running arbitrary commands locally or on remote 
# hosts and reporting their success or failure, typically used from via rake.
#
# = SYNOPSIS
#
#   # Typically used from via rake e.g.
#   require 'triage'
#   include Triage
#
#   HOSTS = %w{ host1 host2 host3 }
#
#   # e.g. run a set of commands locally
#   task :ping do
#     banner 'Ping Hosts'
#     HOSTS.each do |h|
#       check_local "check_icmp -c 400,40% #{h}", "#{h} up"
#     end
#   end
#
#   # e.g. run a command on a set of remote hosts
#   task :disk do
#     banner 'Disk Space'
#     check_by_ssh HOSTS, "check_disk -w 5% -c 2%", "disk space"
#   end
#
module Triage
  # Run the given cmd locally and report success or failure (synchronous)
  def check_local(cmd, label)
    cmd_list = cmd.split(/ +/)
    output = %x{#{cmd}}
    report(label, $?, output)
  end

  # Run cmd on each host in hostlist and report success or failure (asynchronous)
  def check_by_ssh(hostlist, cmd, label = nil, options = {})
    sshr_exec(hostlist, cmd) do |result|
      report(result.host + ' ' + label, result.exit_code, result.stderr)
    end
  end

  # Process the given list of [ host, cmd, [label] ] entries, run the cmd on each
  # host, and report success or failure for each (asynchronous)
  def check_by_ssh_list(list, options = {})
    raise ArgumentError, "Invalid argument - not an Array" unless list.is_a? Array

    # Assemble host_cmd_list and label table
    host_cmd_list = []
    label = {}
    list.each do |entry|
      host_cmd_list.push [ entry[0], entry[1] ]
      label["#{entry[0]} #{entry[1]}"] = entry[2]
    end

    sshr_exec_list(host_cmd_list, options) do |result|
      l = label["#{result.host} #{result.cmd}"] || "#{result.host} #{result.cmd}"
      report(l, result.exit_code, result.stderr)
    end
  end

  # Output a section heading banner
  def banner(label)
    puts
    print bold, yellow, label, reset, "\n"
  end

  private

  def report(label, return_code, error = '')
    if return_code == 0
      status = 'ok'
      colour = green
    else
      status = 'error'
      colour = red
    end
    printf "%-72s %s%s%s%s\n", label, colour, bold, status, reset
    puts '    ' + error if return_code != 0 and error != ''
  end
end

